@@ -1,39 +1,35 @@
 #ifndef G_BASE_DEFINED
 #define G_BASE_DEFINED
-#define GCLIB_VERSION "0.11.7"
+#define GCLIB_VERSION "0.12.6"
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
-#include <string.h>
-#include <stdlib.h>
-#include <stdio.h>
-#include <math.h>
-#include <limits.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <stdint.h>
-#include <stdarg.h>
 
-#if defined __WIN32 || defined __WIN32__ || defined _WIN32 || defined _WIN32_ || defined _WINDOWS
-  #ifndef __WIN32__
-    #define __WIN32__
+#if defined(__WIN32) || defined(__WIN32__) || defined(_WIN32) || defined(_WIN64) || defined(__MINGW64__) || defined(__WINDOWS__)
+  #ifndef _WIN32
+    #define _WIN32
   #endif
+  #ifndef _WIN64
+    #define _WIN64
+  #endif
+  #define __USE_MINGW_ANSI_STDIO 1
+  //#define __ISO_C_VISIBLE 1999
+#endif
+
+#define XSTR(x) STR(x)
+#define STR(x) #x
+
+#ifdef _WIN32
   #include <windows.h>
-  #include <direct.h>
   #include <io.h>
-  #ifndef strcasecmp
-      #define strcasecmp _stricmp
-  #endif
-  #ifndef strncasecmp
-      #define strncasecmp _strnicmp
-  #endif
   #define CHPATHSEP '\\'
   #undef off_t
   #define off_t int64_t
   #ifndef popen
    #define popen _popen
   #endif
+  /*
   #ifndef fseeko
 		#ifdef _fseeki64
 			#define fseeko(stream, offset, origin) _fseeki64(stream, offset, origin)
@@ -41,38 +37,53 @@
 			#define fseeko fseek
 		#endif
   #endif
- #ifndef ftello
-  #ifdef _ftelli64
-    #define ftello(stream) _ftelli64(stream)
-  #else
-    #define ftello ftell
-  #endif
+  #ifndef ftello
+    #ifdef _ftelli64
+      #define ftello(stream) _ftelli64(stream)
+    #else
+      #define ftello ftell
+    #endif
  #endif
+ */
  #else
   #define CHPATHSEP '/'
+  #ifdef __CYGWIN__
+    #define _BSD_SOURCE
+  #endif
   #include <unistd.h>
 #endif
 
-#ifndef fseeko
- #define fseeko fseek
-#endif
-#ifndef ftello
- #define ftello ftell
-#endif
-
 #ifdef DEBUG
 #undef NDEBUG
 #define _DEBUG 1
 #define _DEBUG_ 1
 #endif
 
+#include <stdlib.h>
+#include <stdio.h>
+#include <string.h>
+
+#include <math.h>
+#include <limits.h>
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <stdint.h>
+#include <stdarg.h>
+#include <type_traits>
+
+typedef int64_t int64;
+typedef uint64_t uint64;
 typedef int32_t int32;
 typedef uint32_t uint32;
 typedef int16_t int16;
 typedef uint16_t uint16;
 
 typedef unsigned char uchar;
-typedef unsigned char byte;
+typedef uint8_t byte;
+typedef unsigned int uint;
+
+typedef void* pointer;
+
 
 #ifndef MAXUINT
 #define MAXUINT ((unsigned int)-1)
@@ -90,9 +101,6 @@ typedef unsigned char byte;
 #define MAX_INT INT_MAX
 #endif
 
-typedef int64_t int64;
-typedef uint64_t uint64;
-
 /****************************************************************************/
 
 #ifndef EXIT_FAILURE
@@ -108,21 +116,32 @@ typedef uint64_t uint64;
 
 //-------------------
 
+#define GEXIT(a) { \
+fprintf(stderr, "Error: "); fprintf(stderr, a); \
+GError("Exiting from line %i in file %s\n",__LINE__,__FILE__); \
+}
+
 // Debug helpers
 #ifndef NDEBUG
  #define GASSERT(exp) ((exp)?((void)0):(void)GAssert(#exp,__FILE__,__LINE__))
+ #define GVERIFY(condition) \
+if (!(condition)) { \
+fprintf(stderr, "Assumption \"%s\"\nFailed in file %s: at line:%i\n", \
+#condition,__FILE__,__LINE__); \
+GEXIT(#condition);}
  #ifdef TRACE
-  #define GTRACE(exp)  (GMessage exp)
+  #define GTRACE(exp)  (GMessage(exp))
  #else
-  #define GTRACE(exp)  ((void)0)
+  #define GTRACE(exp)
  #endif
 #else
- #define GASSERT(exp) ((void)0)
- #define GTRACE(exp)  ((void)0)
+ #define GASSERT(exp)
+ #define GTRACE(exp)
+ #define GVERIFY(condition)
 #endif
 
-#define GERROR(exp) (GError exp)
-/**********************************  Macros  ***********************************/
+#define GERROR(exp) (GError(exp))
+
 // Abolute value
 #define GABS(val) (((val)>=0)?(val):-(val))
 
@@ -142,9 +161,6 @@ typedef uint64_t uint64;
 // Clamp value x to range [lo..hi]
 #define GCLAMP(lo,x,hi) ((x)<(lo)?(lo):((x)>(hi)?(hi):(x)))
 
-typedef void* pointer;
-typedef unsigned int uint;
-
 typedef int GCompareProc(const pointer item1, const pointer item2);
 typedef long GFStoreProc(const pointer item1, FILE* fstorage); //for serialization
 typedef pointer GFLoadProc(FILE* fstorage); //for deserialization
@@ -154,6 +170,7 @@ typedef void GFreeProc(pointer item); //usually just delete,
 
 #define GMALLOC(ptr,size)  if (!GMalloc((pointer*)(&ptr),size)) \
                                      GError(ERR_ALLOC)
+
 #define GCALLOC(ptr,size)  if (!GCalloc((pointer*)(&ptr),size)) \
                                      GError(ERR_ALLOC)
 #define GREALLOC(ptr,size) if (!GRealloc((pointer*)(&ptr),size)) \
@@ -175,6 +192,9 @@ inline int iround(double x) {
 int Gmkdir(const char *path, bool recursive=true, int perms = (S_IRWXU | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH));
 void Gmktempdir(char* templ);
 
+
+bool haveStdInput(); //if stdin is from a pipe or redirection
+
 /****************************************************************************/
 
 inline int Gintcmp(int a, int b) {
@@ -196,14 +216,29 @@ template<class T> void Gswap(T& lhs, T& rhs) {
  rhs=tmp;
 }
 
+// use std::is_pointer from <type_traits> in C++11 instead
+/*
+template<typename T>
+  struct isPointer { static const bool value = false; };
+
+template<typename T>
+  struct isPointer<T*> { static const bool value = true; };
+*/
+//check if type T is resolved as a pointer to char
+template<class T>
+  struct is_char_ptr : std::integral_constant <
+      bool,
+      std::is_same<char const *, typename std::decay<T>::type>::value ||
+        std::is_same<char *, typename std::decay<T>::type>::value
+  > {};
 
 /**************** Memory management ***************************/
 
 bool GMalloc(pointer* ptr, unsigned long size); // Allocate memory
 bool GCalloc(pointer* ptr, unsigned long size); // Allocate and initialize memory
 bool GRealloc(pointer* ptr,unsigned long size); // Resize memory
-void GFree(pointer* ptr); // Free memory, resets ptr to NULL
 
+void GFree(pointer* ptr); // Free memory, resets ptr to NULL
 
 //int saprintf(char **retp, const char *fmt, ...);
 
@@ -212,6 +247,15 @@ void GMessage(const char* format,...);// Log message to stderr
 // Assert failed routine:- usually not called directly but through GASSERT
 void GAssert(const char* expression, const char* filename, unsigned int lineno);
 
+
+template<class T> T* GDupAlloc(T& data) {
+	T* tmp=NULL;
+	if (!GMalloc((pointer*) tmp, sizeof(T)))
+			GError(ERR_ALLOC);
+	memcpy((void*)tmp, (void*)&data, sizeof(T));
+	return tmp;
+}
+
 // ****************** basic string manipulation *************************
 char *Gstrdup(const char* str, int xtracap=0); //string duplication with extra capacity added
 //duplicate a string by allocating a copy for it (+xtracap heap room) and returning the new pointer
@@ -233,6 +277,7 @@ char* loCase(const char* str);
 char* strlower(char * str);
 char* strupper(char * str);
 
+
 //strstr but for memory zones: scans a memory region
 //for a substring:
 void* Gmemscan(void *mem, unsigned int len,
@@ -289,12 +334,10 @@ int strhash(const char* str);
 //alternate hash functions:
 int fnv1a_hash(const char* cp);
 int djb_hash(const char* cp);
-int murmur3(const char *key);
 
 //---- generic base GSeg : genomic segment (interval) --
 // coordinates are considered 1-based (so 0 is invalid)
-class GSeg {
- public:
+struct GSeg {
   uint start; //start<end always!
   uint end;
   GSeg(uint s=0,uint e=0) {
@@ -304,17 +347,14 @@ class GSeg {
   //check for overlap with other segment
   uint len() { return end-start+1; }
   bool overlap(GSeg* d) {
-     //return start<d->start ? (d->start<=end) : (start<=d->end);
      return (start<=d->end && end>=d->start);
   }
 
   bool overlap(GSeg& d) {
-     //return start<d.start ? (d.start<=end) : (start<=d.end);
      return (start<=d.end && end>=d.start);
   }
 
   bool overlap(GSeg& d, int fuzz) {
-     //return start<d.start ? (d.start<=end+fuzz) : (start<=d.end+fuzz);
      return (start<=d.end+fuzz && end+fuzz>=d.start);
   }
 
@@ -389,6 +429,21 @@ class GSeg {
      }
 };
 
+struct GRangeParser: GSeg {
+	char* refName=NULL;
+	char strand=0;
+	void parse(char* s);
+	GRangeParser(char* s=NULL):GSeg(0, 0) {
+		if (s) parse(s);
+	}
+	~GRangeParser() {
+		GFREE(refName);
+	}
+};
+
+
+
+
 //basic dynamic array template for primitive types
 //which can only grow (reallocate) as needed
 
@@ -437,7 +492,7 @@ template<class OBJ> class GDynArray {
     		Clear();
     		return *this;
     	}
-    	increaseCapacity(a.fCapacity); //set size
+    	growTo(a.fCapacity); //set size
         memcpy(fArray, a.fArray, sizeof(OBJ)*a.fCount);
         return *this;
     }
@@ -452,7 +507,7 @@ template<class OBJ> class GDynArray {
     	if (GDynArray_MAXCOUNT-delta<=fCapacity)
     		delta=GDynArray_MAXCOUNT-fCapacity;
     	if (delta<=1) GError("Error at GDynArray::Grow(): max capacity reached!\n");
-    	increaseCapacity(fCapacity + delta);
+    	growTo(fCapacity + delta);
     }
 #define GDynArray_ADD(item) \
     	if (fCount==MAX_UINT-1) GError("Error at GDynArray: cannot add item, maximum count reached!\n"); \
@@ -484,12 +539,27 @@ template<class OBJ> class GDynArray {
 
     uint Count() { return fCount; } // get size of array (elements)
     uint Capacity() { return fCapacity; }
-    void increaseCapacity(uint newcap) {
+    void growTo(uint newcap) {
     	if (newcap==0) { Clear(); return; }
-    	if (newcap <= fCapacity) return; //never shrinks (use Pack() for this)
+    	if (newcap <= fCapacity) return; //never shrink! (use Pack() for shrinking)
     	GREALLOC(fArray, newcap*sizeof(OBJ));
     	fCapacity=newcap;
     }
+
+    void append(OBJ* arr, uint count) {
+    	//fast adding of a series of objects
+    	growTo(fCount+count);
+    	memcpy(fArray+fCount, arr, count*sizeof(OBJ));
+    	fCount+=count;
+    }
+
+    void append(GDynArray<OBJ> arr) {
+    	//fast adding of a series of objects
+    	growTo(fCount+arr.fCount);
+    	memcpy(fArray+fCount, arr.fArray, arr.fCount*sizeof(OBJ));
+    	fCount+=arr.fCount;
+    }
+
     void Trim(int tcount=1) {
     	//simply cut (discard) the last tcount items
     	//new Count is now fCount-tcount
@@ -504,6 +574,15 @@ template<class OBJ> class GDynArray {
     	fCapacity=newcap;
     }
 
+    void zPack(OBJ z) { //shrink capacity to fCount+1 and adds a z terminator
+    	if (fCapacity-fCount<=1) { fArray[fCount]=z; return; }
+    	int newcap=fCount+1;
+    	GREALLOC(fArray, newcap*sizeof(OBJ));
+    	fCapacity=newcap;
+    	fArray[fCount]=z;
+    }
+
+
     inline void Shrink() { Pack(); }
 
     void Delete(uint idx) {
@@ -528,7 +607,7 @@ template<class OBJ> class GDynArray {
 
     OBJ* operator()() { return fArray; }
 
-    //use below to prevent freeing the fArray pointer
+    //use methods below in order to prevent deallocation of fArray pointer on destruct
     //could be handy for adopting stack objects (e.g. cheap dynamic strings)
     void ForgetPtr() { byptr=true;  }
     void DetachPtr() { byptr=true;  }
@@ -548,7 +627,6 @@ int strsplit(char* str, GDynArray<char*>& fields, int maxfields=MAX_INT); //spli
 //splits a string by placing 0 where tab or space is found, setting fields[] to the beginning
 //of each field (stopping after maxfields); returns number of fields parsed
 
-//--------------------------------------------------------
 // ************** simple line reading class for text files
 //GLineReader -- text line reading/buffering class
 class GLineReader {
