@@ -11,7 +11,7 @@
 #include "proc_mem.h"
 #endif
 
-#define VERSION "2.1.4"
+#define VERSION "2.1.6"
 
 //#define DEBUGPRINT 1
 
@@ -34,10 +34,12 @@ stringtie <in.bam ..> [-G <guide_gff>] [-l <prefix>] [-o <out.gtf>] [-p <cpus>]\
  [-v] [-a <min_anchor_len>] [-m <min_len>] [-j <min_anchor_cov>] [-f <min_iso>]\n\
  [-c <min_bundle_cov>] [-g <bdist>] [-u] [-L] [-e] [--viral] [-E <err_margin>]\n\
  [--ptf <f_tab>] [-x <seqid,..>] [-A <gene_abund.out>] [-h] {-B|-b <dir_path>}\n\
+ [--mix] [--conservative] [--rf] [--fr]\n\
 Assemble RNA-Seq alignments into potential transcripts.\n\
 Options:\n\
  --version : print just the version at stdout and exit\n\
  --conservative : conservative transcript assembly, same as -t -c 1.5 -f 0.05\n\
+ --mix : both short and long read data alignments are provided\n\
  --rf : assume stranded library fr-firststrand\n\
  --fr : assume stranded library fr-secondstrand\n\
  -G reference annotation to use for guiding the assembly process (GTF/GFF3)\n\
@@ -133,6 +135,7 @@ GStr out_dir;
 GStr tmp_path;
 GStr tmpfname;
 GStr genefname;
+GStr traindir; // training directory for CDS option
 bool guided=false;
 bool trim=true;
 bool viral=false;
@@ -161,6 +164,7 @@ uint runoffdist=200;
 float mcov=1; // fraction of bundle allowed to be covered by multi-hit reads paper uses 1
 int allowed_nodes=1000;
 //bool adaptive=true; // adaptive read coverage -> depends on the overall gene coverage
+//GPVec<CDSparam> cds;
 
 int no_xs=0; // number of records without the xs tag
 
@@ -195,6 +199,8 @@ bool forceBAM = false; //useful for stdin (piping alignments into StringTie)
 bool mergeMode = false; //--merge option
 bool keepTempFiles = false; //--keeptmp
 
+bool mixedMode = false; // both short and long read data alignments are provided
+
 int GeneNo=0; //-- global "gene" counter
 double Num_Fragments=0; //global fragment counter (aligned pairs)
 double Frag_Len=0;
@@ -243,7 +249,7 @@ GFastMutex countMutex;
 
 #endif
 
-GHash<int> excludeGseqs; //hash of chromosomes/contigs to exclude (e.g. chrM)
+GStrSet<> excludeGseqs; //hash of chromosomes/contigs to exclude (e.g. chrM)
 
 bool NoMoreBundles=false;
 bool moreBundles(); //thread-safe retrieves NoMoreBundles
@@ -277,7 +283,7 @@ int main(int argc, char* argv[]) {
 
  // == Process arguments.
  GArgs args(argc, argv,
-   "debug;help;version;viral;conservative;keeptmp;rseq=;ptf=;bam;fr;rf;merge;"
+   "debug;help;version;viral;conservative;mix;cds=;keeptmp;rseq=;ptf=;bam;fr;rf;merge;"
    "exclude=zihvteuLRx:n:j:s:D:G:C:S:l:m:o:a:j:c:f:p:g:P:M:Bb:A:E:F:T:");
  args.printError(USAGE, true);
 
@@ -429,16 +435,18 @@ if (ballgown)
 #ifndef NOTHREADS
 //model: one producer, multiple consumers
 #define DEF_TSTACK_SIZE 8388608
+ size_t defStackSize=DEF_TSTACK_SIZE;
+#ifdef _GTHREADS_POSIX_
  int tstackSize=GThread::defaultStackSize();
- size_t defStackSize=0;
-if (tstackSize<DEF_TSTACK_SIZE) defStackSize=DEF_TSTACK_SIZE;
+ if (tstackSize<DEF_TSTACK_SIZE) defStackSize=DEF_TSTACK_SIZE;
  if (verbose) {
-	 if (defStackSize>0){
-		 int ssize=defStackSize;
-		 GMessage("Default stack size for threads: %d (increased to %d)\n", tstackSize, ssize);
-	 }
-	 else GMessage("Default stack size for threads: %d\n", tstackSize);
+   if (defStackSize>0){
+    int ssize=defStackSize;
+    GMessage("Default stack size for threads: %d (increased to %d)\n", tstackSize, ssize);
+   }
+   else GMessage("Default stack size for threads: %d\n", tstackSize);
  }
+#endif
  GThread* threads=new GThread[num_cpus]; //bundle processing threads
 
  GPVec<BundleData> bundleQueue(false); //queue of loaded bundles
@@ -958,7 +966,11 @@ void processOptions(GArgs& args) {
 		 bundledist=0;
 		 singlethr=1.5;
 	 }
-
+	 mixedMode=(args.getOpt("mix")!=NULL);
+	 if(mixedMode) {
+		 bundledist=0;
+		 //isofrac=0.02; // allow mixedMode to be more conservative
+	 }
 
 	if (args.getOpt("conservative")) {
 	  isofrac=0.05;
@@ -1023,13 +1035,19 @@ void processOptions(GArgs& args) {
 		 gfasta=new GFastaDb(s.chars());
 	 }
 
+	 /*traindir=args.getOpt("cds");
+	 if(!traindir.is_empty()) {
+		 if(gfasta==NULL) GError("Genomic sequence file is required for --cds option.\n");
+		 load_cds_param(traindir,cds);
+	 }*/
+
      s=args.getOpt('x');
      if (!s.is_empty()) {
     	 //split by comma and populate excludeGSeqs
     	 s.startTokenize(" ,\t");
     	 GStr chrname;
     	 while (s.nextToken(chrname)) {
-    		 excludeGseqs.Add(chrname.chars(),new int(0));
+    		 excludeGseqs.Add(chrname.chars());
     	 }
      }
 
@@ -1068,12 +1086,17 @@ void processOptions(GArgs& args) {
 
 	 rawreads=(args.getOpt('R')!=NULL);
 	 if(rawreads) {
+		 if(mixedMode) {
+			 GError("Mixed mode and rawreads options are incompatible!\n");
+		 }
+
 		 if(!longreads) {
 			 if(verbose) GMessage("Enable longreads processing\n");
 			 longreads=true;
 			 bundledist=0;
 		 }
 		 readthr=0;
+
 	 }
 
 	 s=args.getOpt('c');
@@ -1258,7 +1281,7 @@ void processOptions(GArgs& args) {
 	 { //prepare temp path
 		 GStr stempl(out_dir);
 		 stempl.chomp('/');
-		 stempl+="/tmp.XXXXXXXX";
+		 stempl+="/tmp_XXXXXX";
 		 char* ctempl=Gstrdup(stempl.chars());
 	     Gmktempdir(ctempl);
 	     tmp_path=ctempl;
@@ -1330,11 +1353,11 @@ void noMoreBundles() {
 		  if (areThreadsWaiting) {
 		    DBGPRINT("##> NOTIFY ALL workers: no more data!\n");
 		    haveBundles.notify_all();
-		    current_thread::sleep_for(10);
+		    current_thread::sleep_for(1);
 		    waitMutex.lock();
 		     areThreadsWaiting=(threadsWaiting>0);
 		    waitMutex.unlock();
-		    current_thread::sleep_for(10);
+		    current_thread::sleep_for(1);
 		  }
 		} while (areThreadsWaiting); //paranoid check that all threads stopped waiting
 #else
@@ -1585,7 +1608,7 @@ int loadPtFeatures(FILE* f, GArray<GRefPtData>& refpts) {
   return num;
 }
 
-void writeUnbundledGenes(GHash<CGene>& geneabs, const char* refseq, FILE* gout) {
+void writeUnbundledGenes(GHash<CGene*>& geneabs, const char* refseq, FILE* gout) {
 				 //write unbundled genes from this chromosome
 	geneabs.startIterate();
 	while (CGene* g=geneabs.NextData()) {
@@ -1604,7 +1627,7 @@ void writeUnbundledGuides(GVec<GRefData>& refdata, FILE* fout, FILE* gout) {
  for (int g=0;g<refdata.Count();++g) {
 	 GRefData& crefd=refdata[g];
 	 if (crefd.rnas.Count()==0) continue;
-	 GHash<CGene> geneabs;
+	 GHash<CGene*> geneabs;
 	 //gene_id abundances (0), accumulating coords
 	 for (int m=0;m<crefd.rnas.Count();++m) {
 		 GffObj &t = *crefd.rnas[m];
